/*********************************************************************
本ソースリストおよびソフトウェアは、ライセンスフリーです。
利用、編集、再配布等が自由に行えますが、著作権表示の改変は禁止します。

なお、Radiation-Watch.orgのソースコードを含みます。

                               Copyright (c) 2012-2013 Wataru KUNINO
                               http://www.geocities.jp/bokunimowakaru/
*********************************************************************/

//////////////////////////////////////////////////
// Radiation-Watch.org
// URL http://www.radiation-watch.org/
//////////////////////////////////////////////////
/*

Radiation-watch.orgプロジェクトは開発成果をクリエイティブ・コモンズ(CC 
BY 3.0)ライセンスの元でオープンソース化しています。以下では、ポケット
ガイガーの回路図、仕様、ソースコード、実験結果を、順次公開します。
これらは全世界において、商用・非商用を問わず自由に再配布・改変・公開す
ることができます。なお一部のコンテンツは英語またはオランダ語のみとなり
ます。

POCKET GEIGER by Radiation-Watch.org is licensed under a Creative 
Commons Attribution 3.0 Unported License.

*/

// include the library code:
#include <Wire.h>
#include <Adafruit_MCP23017.h>
#include <Adafruit_RGBLCDShield.h>

Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();

///　デジタルIOのピンの設定　///
int signPin = 2; //信号(黄)
int noisePin = 5; //ノイズ(緑)
//VCC(赤)
//GND(青)
////////////////////////////////

const double alpha=53.032; // cpm = uSv × alpha

int index=0; //ループを回った回数
char msg[256]=""; //シリアルで出力する文字列用

int signCount=0;  //信号のカウント回数
int noiseCount=0;  //ノイズのカウント回数

int sON=0;//信号検出時のロックフラグ
int nON=0;//ノイズ検出時のロックフラグ

double cpm = 0; //現在のカウント数
double cpmHistory[200]; //カウント回数の履歴
int cpmIndex=0;//カウント回数の現在配列の格納位置
int cpmIndexPrev=0;//二重カウントの防止用

//ループの時間間隔の計測用
int prevTime=0;
int currTime=0; 

int totalSec=0; //計測開始からのトータルの時間(秒)
int totalHour=0; //計測開始からのトータルの時間(時間)

//cpm計算用の時間
int cpmTimeMSec=0;
int cpmTimeSec=0;
int cpmTimeMin=0;

//シリアルで出力するときの浮動少数を文字列に変換用
char cpmBuff[20];
char uSvBuff[20];
char uSvdBuff[20];

void setup(){
	lcd.begin(16, 2);		//液晶の開始
	lcd.clear();
	Serial.begin(9600);  	//シリアルの準備  //9600bps
	pinMode(signPin,INPUT);	//信号検出ピンの設定
	digitalWrite(signPin,HIGH);
	pinMode(noisePin,INPUT);	//ノイズ検出ピンの設定
	digitalWrite(noisePin,HIGH);
	//CSV形式でシリアルで出力するときの列の名前を列挙(_を,に読み替える)
	Serial.println("hour[h]_sec[s]_count_cpm_uSv/h_uSv/hError");
	//cpmの履歴を保存する配列の初期化
	for(int i=0; i<200;i++ ){
		cpmHistory[i]=0;
	}
	//ループ開始時間を取得
	prevTime = millis();
}

void loop(){
	int sign = digitalRead(signPin);  // 信号のローデータ　通常:High  検出時:Low
	int noise = digitalRead(noisePin);  // ノイズのローデータ　通常:Low  検出時:High

	if(sign==0 && sON==0){  //信号の検出(100us程度 Lowになる)。しばらく続くのでその間はカウントしないようにする
		sON = 1;		//信号検出時のロックフラグを立てる
		signCount++;	//放射線パルスをカウント
	}else if(sign==1 && sON==1){   //信号が終了したのでフラグを解除
		sON = 0;		//信号検出時のロックフラグを下げる
	}
	if(noise==1 && nON==0){  //ノイズの検出(100us程度 Lowになる)、しばらく続くのでその間はカウントしないようにする
		nON = 1;		//ノイズ検出時のロックフラグを立てる
		noiseCount++;
	}else if(noise==0 && nON==1){   //ノイズが終了したのでフラグを解除
		nON = 0;
	}
	//10000回ループを回ったら、計測値を計算して、シリアルで出力する
	if(index==10000){ //Arduino Nano(ATmega328)で160-170ms程度
		currTime = millis();    //現在の時刻を取得
		if(noiseCount == 0){		//ノイズが10000回のループの中でまったく検出されないとき（ノイズが検出されたときは、処理をしない）
			if( totalSec % 6 == 0 && cpmIndexPrev != totalSec){			//6秒ごとにカウント回数の履歴を格納する配列をずらす
				cpmIndexPrev = totalSec;
				cpmIndex++;
				if(cpmIndex >= 200){        //最後まできたら元に戻す
					cpmIndex = 0;
				}
				//一周してきたときに次に格納する配列に値が詰まっていれば、
				//現在の合計しているカウント(変数cpm)から引いて、無かったことにする
				if(cpmHistory[cpmIndex] > 0){
					cpm -= cpmHistory[cpmIndex];
				}
				cpmHistory[cpmIndex]=0;
			}
			cpmHistory[cpmIndex] += signCount;      //カウント回数の履歴を保存
			cpm += signCount;      //カウント回数の蓄積
			cpmTimeMSec += abs(currTime - prevTime);      //10000回、データを取得するのにかかった時間を取得
			if(cpmTimeMSec >= 1000){      //ms→secに変換する(オーバーフロー対策)
				cpmTimeMSec -= 1000;
				if( cpmTimeSec >= 20*60 ){        //cpmを求めるときに使用する計測時間(最大20分)を加算
					cpmTimeSec = 20*60;
				}else{
					cpmTimeSec++;
				}
				totalSec++;        //トータルでの計測時間
				if(totalSec >= 3600){        //sec→hourに変換する(オーバーフロー対策)
					totalSec -= 3600;
					totalHour++;
				}
			}
			double min = cpmTimeSec / 60.0;      //現在の計測時間(最大20分)
			if(min!=0){        //cpm、uSv/h、uSv/hの誤差をそれぞれ計算する
				dtostrf(cpm / min, -1, 2, cpmBuff);
				dtostrf(cpm / min / alpha, -1, 2, uSvBuff);
				dtostrf(sqrt(cpm) / min / alpha, -1, 2, uSvdBuff);
			}else{        //0割り算のときは、0にする
				dtostrf(0, -1, 2, cpmBuff);
				dtostrf(0, -1, 2, uSvBuff);
				dtostrf(0, -1, 2, uSvdBuff);
			}
			//液晶への表示
			char time_s[6] = "00:00";
			time_s[0] = (char)(totalSec/600)+'0';
			time_s[1] = (char)((totalSec/60)%10)+'0';
			time_s[3] = (char)((totalSec%60)/10)+'0';
			time_s[4] = (char)((totalSec%60)%10)+'0';
			lcd.setCursor(0,0);
			lcd.print( cpmBuff );
			lcd.print(" cpm   ");
			lcd.setCursor(11,0);
			lcd.print( time_s );
			lcd.print(" ");
			lcd.setCursor(0,1);
			lcd.print( uSvBuff );
			lcd.print(" uSv/h      ");
			
			//シリアルで送信する文字列の生成
			sprintf(msg, "%d,%d.%03d,%d,%s,%s,%s",
				totalHour,totalSec,
				cpmTimeMSec,
				signCount,
				cpmBuff,
				uSvBuff,
				uSvdBuff
			);
			Serial.println(msg);      //シリアルで送信
			index=0;
		}
		//次の10000回のループのための変数の初期化
		prevTime = currTime;
		signCount=0;
		noiseCount=0;
	}
	index++;
}
